unit SyntaxWizard;

// Set the syntax highlighter for an edit buffer.
// This wizard adds a Syntax menu item to the Edit menu in Delphi's IDE.
// This menu item has submenu items for each kind of syntax highlighting
// that Delphi supports. The user can choose a menu item to set the syntax
// highlighting for a buffer.
//
// Copyright  1999 Tempest Software, Inc.

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Menus, Dialogs, ToolsAPI;

type
  TSyntaxWizard = class(TInterfacedObject, IOTANotifier, IOTAWizard)
  private
    { Private declarations }
    SyntaxMenu: TMenuItem;
    function GetCurrentEditor: IOTASourceEditor;
    function CreateSyntaxMenu: TMenuItem;
    procedure FillSyntaxMenu(Menu: TMenuItem);
    procedure ItemClick(Sender: TObject);
    procedure MenuClick(Sender: TObject);

    procedure AfterSave;  // IOTANotifier stubs
    procedure BeforeSave;
    procedure Destroyed;
    procedure Modified;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Execute;
    function GetIDString: string;
    function GetName: string;
    function GetState: TWizardState;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterPackageWizard(TSyntaxWizard.Create);
end;

constructor TSyntaxWizard.Create;
begin
  SyntaxMenu := CreateSyntaxMenu;
end;

destructor TSyntaxWizard.Destroy;
begin
  SyntaxMenu.Free;
  inherited;
end;

function TSyntaxWizard.CreateSyntaxMenu: TMenuItem;
resourcestring
  sCaption = 'Synta&x';
var
  Services: INTAServices;
  MainMenu: TMainMenu;
  EditMenu: TMenuItem;
  I: Integer;
begin
  Services := BorlandIDEServices as INTAServices;
  MainMenu := Services.MainMenu;
  EditMenu := nil;
  for I := 0 to MainMenu.Items.Count-1 do
    if CompareText(MainMenu.Items[I].Name, 'EditMenu') = 0 then
    begin
      EditMenu := MainMenu.Items[I];
      Break;
    end;
  Assert(EditMenu <> nil, 'Cannot find Edit menu');

  Result := nil;
  for I := 0 to EditMenu.Count-1 do
    if CompareText(EditMenu.Items[I].Name, 'EditSelectAll') = 0 then
    begin
      Result := NewItem(sCaption, 0, False, True, MenuClick, 0, '');
      EditMenu.Insert(I+1, Result); // after Edit>Select All;
      Break;
    end;
  Assert(Result <> nil, 'Cannot find Edit>Select All menu item');

  FillSyntaxMenu(Result);
end;

procedure TSyntaxWizard.FillSyntaxMenu(Menu: TMenuItem);
resourcestring
  sNone = '&None';
  sPascal = '&Pascal';
  sC = '&C/C++';
  sSQL = '&SQL';

  procedure AddItem(const Caption: string; const Syntax: TOTASyntaxHighlighter);
  var
    Item: TMenuItem;
  begin
    Item := NewItem(Caption, 0, False, False, ItemClick, 0, '');
    Item.Tag := Ord(Syntax);
    Item.GroupIndex := 1;
    Item.RadioItem := True;
    Menu.Add(Item);
  end;

begin
  AddItem(sNone, shNone);
  AddItem(sPascal, shPascal);
  AddItem(sC, shC);
  AddItem(sSQL, shSQL);
end;

procedure TSyntaxWizard.MenuClick(Sender: TObject);
var
  Enable: Boolean;
  I: Integer;
  Item: TMenuItem;
  Editor: IOTASourceEditor;
  Syntax: TOTASyntaxHighlighter;
begin
  Editor := GetCurrentEditor;
  Enable := Editor <> nil;
  if Enable then
    Syntax := Editor.SetSyntaxHighlighter(shQuery)
  else
    Syntax := shNone;
  for I := 0 to SyntaxMenu.Count-1 do
  begin
    Item := SyntaxMenu.Items[I];
    Item.Enabled := Enable;
    Item.Checked := Enable and (Item.Tag = Ord(Syntax));
  end;
end;

procedure TSyntaxWizard.ItemClick(Sender: TObject);
var
  Editor: IOTASourceEditor;
begin
  Editor := GetCurrentEditor;
  Assert(Editor <> nil);
  with Sender as TMenuItem do
  begin
    Checked := True;
    Editor.SetSyntaxHighlighter(TOTASyntaxHighlighter(Tag));
  end;
end;

// Return the editor for the current module.
function TSyntaxWizard.GetCurrentEditor: IOTASourceEditor;
var
  Module: IOTAModule;
  I: Integer;
begin
  Module := (BorlandIDEServices as IOTAModuleServices).CurrentModule;
  if Module <> nil then
    for I := 0 to Module.GetModuleFileCount-1 do
      if Module.GetModuleFileEditor(I).QueryInterface(IOTASourceEditor, Result) = S_OK then
        Exit;
  Result := nil;
end;


// Stub: Delphi never calls Execute for this wizard.
procedure TSyntaxWizard.Execute;
begin
end;

function TSyntaxWizard.GetIDString: string;
begin
  Result := 'Tempest Software.Syntax Highlighter Wizard';
end;

function TSyntaxWizard.GetName: string;
begin
  Result := 'Syntax Highlighter Wizard';
end;

function TSyntaxWizard.GetState: TWizardState;
begin
  Result := [];
end;

procedure TSyntaxWizard.AfterSave;
begin
end;

procedure TSyntaxWizard.BeforeSave;
begin
end;

procedure TSyntaxWizard.Destroyed;
begin
end;

procedure TSyntaxWizard.Modified;
begin
end;

end.
